"use strict";

jQuery(function($) {

    // on body when vixai-auto-writer-nav is clicked
    $("body").on("click", ".vixai-auto-writer-nav a", function (event) {
        event.preventDefault();
        // parse the href parameters and get the paged value
        let url = new URL($(this).attr('href'));
        let paged = url.searchParams.get("paged");

        loadPostPage(paged);
    });

    $('#vixai-auto-writer-schedule').on('click', function () {
        toggleScheduleOptions();
        return false;
    });

    $('#vixai-auto-writer-cancel-schedule').on('click', function () {
        toggleScheduleOptions();
        return false;
    });

    $('#vixai-auto-writer-confirm-schedule').on('click', function () {
        submitForm($(this), true);
    });

    $("#vixai-auto-writer-form").submit(function (event) {
        event.preventDefault();

        submitForm($("#vixai-auto-writer-generate"), false);
    });

    $("#vixai-auto-writer-save").on('click', function () {
        submitForm($(this), true, true);
    });

    $(".vixai-scheduler-generators-delete").on('click', function (event) {
        event.preventDefault();
        if (confirm($(this).data('confirm-message'))) {
            let deleteUrl = $(this).attr('href');
            $.ajax({
                type: "POST",
                url: deleteUrl,
                data: JSON.stringify({
                    id: $(this).data('id'),
                }),
                dataType: "json",
                encode: true,
                headers: {
                    'Content-Type': 'application/json',
                    'X-WP-Nonce': vixai.nonce,
                },
            }).success(function (response) {
                // refresh the page
                location.reload();
            }).fail(function (response) {
                alert('Error: ' + response.responseText);
            });
        }
    });

    const submitForm = function (button, isScheduled = false, isEdit = false) {
        if (!isProperlyConfigured()) {
            alert('Please enter a valid OpenAI API key in the settings page.');
            return;
        }

        if ($("#vixai-auto-writer-topic").val() === '') {
            alert($('#vixai-auto-writer-topic').data('validation-message'));
            return;
        }

        // add spinner to the button #vixai-auto-writer-generate bootstrap spinner
        button.prepend('<span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>');
        // disable the button
        button.prop('disabled', true);

        if (!isScheduled) {
            $(".vixai-dont-close-page").show();
        }

        let prompts = {};
        $(".vixai-auto-writer-prompt").each(function () {
            prompts[$(this).data('prompt-id')] = $(this).val();
        });

        let formData = {
            topic: $("#vixai-auto-writer-topic").val(),
            include_outline: $("#vixai-auto-writer-include-outline").is(':checked'),
            include_featured_image: $("#vixai-auto-writer-include-featured-image").is(':checked'),
            include_section_images: $("#vixai-auto-writer-include-section-images").is(':checked'),
            include_tldr: $("#vixai-auto-writer-include-tldr").is(':checked'),
            include_conclusion: $("#vixai-auto-writer-include-conclusion").is(':checked'),
            post_type: $("#vixai-auto-writer-post-type").val(),
            post_status: $("#vixai-auto-writer-post-status").val(),
            post_category: $("#vixai-auto-writer-post-category").val(),
            number_of_sections: $("#vixai-auto-writer-sections").val(),
            section_max_length: $("#vixai-auto-writer-words-per-section").val(),
            number_of_articles: $("#vixai-auto-writer-articles").val(),
            seo_keywords: $("#vixai-auto-writer-seo-keywords").val(),
            prompts: prompts,
        };

        if (isScheduled) {
            formData.interval = $("#vixai-auto-writer-schedule-interval").val();
            formData.max_runs = $("#vixai-auto-writer-max-runs").val();
        }

        if (isEdit) {
            formData.id = $("#vixai-auto-writer-generator-id").val();
            formData.active = $("#vixai-auto-writer-generator-status").val() === "1";
            formData.max_runs = $("#vixai-auto-writer-max-runs").val();
        }

        let url = $('#vixai-auto-writer-form').attr('action');
        if (isScheduled) {
            url = url + '&scheduled=1';
        }

        $.ajax({
            type: "POST",
            url: url,
            data: JSON.stringify(formData),
            dataType: "json",
            encode: true,
            headers: {
                'Content-Type': 'application/json',
                'X-WP-Nonce': vixai.nonce,
            },
        }).success(function (response) {
            button.find('.spinner-border').remove();
            button.prop('disabled', false);

            if (!isScheduled) {
                refreshPosts($("#vixai-auto-writer-articles").val());
                $(".vixai-dont-close-page").hide();
            } else {
                toggleScheduleOptions();

                if (!isEdit) {
                    showToast(
                        translate('Scheduled'),
                        translate('AI Auto Writer') +
                        ' <a href="' + response.url + '">' +
                        translate('scheduled Successfully.') +
                        '</a>');
                }
            }

        }).fail(function (response) {
            alert('Error: ' + response.responseText);
            button.find('.spinner-border').remove();
            button.prop('disabled', false);

            if (!isScheduled) {
                $(".vixai-dont-close-page").hide();
            }
        });
    }

    const translate = function (text) {
        // get translations from the #vixai-auto-writer-translations input
        let translations = JSON.parse($('#vixai-auto-writer-translations').val());
        if (translations[text]) {
            return translations[text];
        }

        return text;
    }

    const showToast = function (title, message) {

        let container = $('<div class="toast-container position-fixed bottom-0 end-0 p-3">');
        let toast = $('<div class="toast" role="alert" aria-live="assertive" aria-atomic="true">');
        let toastHeader = $('<div class="toast-header">');
        toastHeader.append('<strong class="me-auto">' + title + '</strong>');
        toastHeader.append('<button type="button" class="btn-close" data-bs-dismiss="toast" aria-label="Close"></button>');
        let toastBody = $('<div class="toast-body">');

        toastBody.append(message);
        toast.append(toastHeader);
        toast.append(toastBody);
        container.append(toast);
        $('body').append(container);

        // show the toast
        let toastEl = new bootstrap.Toast(toast);
        toastEl.show();
    }

    const refreshPosts = function (number_of_articles) {
        loadPostPage(1, function () {
            $("#vixai-auto-writer-posts tbody tr").slice(0, number_of_articles).css('background-color', '#eff4fb');
            $("#vixai-auto-writer-posts tbody tr").slice(0, number_of_articles).animate({backgroundColor: '#fff'}, 3000);
        });
    }

    const toggleScheduleOptions = function () {
        $("#vixai-auto-writer-generate").toggle('fast');

        $(".row.vixai-schedule-options").toggle(100);

        $('#vixai-auto-writer-schedule').toggle(100);
    }

    const loadPostPage = function (paged, onSuccess) {
        $.ajax({
            type: "GET",
            url: vixai.siteUrl + '/?rest_route=/vixai/auto-writer/v1/list&paged=' + paged,
            dataType: "json",
            encode: true,
            headers: {
                'Content-Type': 'application/json',
                'X-WP-Nonce': vixai.nonce,
            },
        }).success(function (response) {
            // replace with new posts
            $("#vixai-auto-writer-posts").html(response.body);

            if (typeof onSuccess === 'function') {
                onSuccess();
            }

        }).fail(function (response) {
            alert('Error: ' + response.responseText);
        });
    }

    const isProperlyConfigured = function () {
        if (vixai.isOpenAIKeyValid === undefined || vixai.isOpenAIKeyValid === "0" || vixai.isOpenAIKeyValid === "" || vixai.isOpenAIKeyValid === false) {
            return false;
        }

        return true;
    }
});
