"use strict";

tinyMCEPreInit.mceInit['elementorwpeditor'].plugins = tinyMCEPreInit.mceInit['elementorwpeditor'].plugins || "";
tinyMCEPreInit.mceInit['elementorwpeditor'].plugins += ",vixai_classic";

tinyMCEPreInit.mceInit['elementorwpeditor'].toolbar1 = tinyMCEPreInit.mceInit['elementorwpeditor'].toolbar1 || "";
tinyMCEPreInit.mceInit['elementorwpeditor'].toolbar1 += ",vixai_classic_button_text";
tinyMCEPreInit.mceInit['elementorwpeditor'].toolbar1 += ",vixai_classic_button_images";

tinyMCEPreInit.qtInit['elementorwpeditor'].buttons = tinyMCEPreInit.qtInit['elementorwpeditor'].buttons || "";
tinyMCEPreInit.qtInit['elementorwpeditor'].buttons += ",vixai_classic_button_text";
tinyMCEPreInit.qtInit['elementorwpeditor'].buttons += ",vixai_classic_button_images";

jQuery( document ).ready(function() {

    jQuery( document ).on( 'DOMNodeInserted', '.mce-tinymce', async function (obj) {

        // if the closets .elementor-controls doesn't have a child with class vixai-editor-is-there then return
        if ( jQuery(obj.currentTarget).closest('#elementor-controls').find('.vixai-editor-is-there').length === 0 ) {
            return;
        }

        const rgb = jQuery('.mce-i-bold').css('color');
        // convert rgb to array
        const rgbArray = rgb.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);

        const color = new Color(rgbArray[1], rgbArray[2], rgbArray[3]);
        const solver = new Solver(color);
        const result = solver.solve();

        // set the filter for any i inside a mce-vixai-classic-main-button
        jQuery('.mce-vixai-classic-main-button i.mce-ico').css('filter', result.filter);

        jQuery('.mce-floatpanel').css('width', 'auto !important')
        jQuery('.mce-container-body').css('width', 'auto !important')

    } );

    elementor.hooks.addAction( 'panel/open_editor/widget/vixai_editor', function( panel, model, view ) {

    } );
});

jQuery( document ).ready(function() {

    tinymce.PluginManager.add('vixai_classic', function(editor, url) {

        if ( jQuery('#vixai-js-configs').length === 0 ) {
            return;
        }

        let vixai = JSON.parse(jQuery('#vixai-js-configs').val());

        if (vixai === null) {
            return;
        }

        let menu = [];

        for (let operation in vixai.prompts) {
            let prompt = vixai.prompts[operation];
            menu.push({
                text: prompt.menuTitle,
                image: vixai.pluginUrl + 'includes/icons/' + prompt.icon + '.svg',
                classes: 'vixai-classic-button',
                onclick: async function () {

                    if (isvixaiProperlyConfigured() === false) {
                        return;
                    }

                    let selectedText = editor.selection.getContent({format: 'text'});
                    if (selectedText === '' && (prompt.requiresTextSelection || prompt.requiresTextSelection === 1 || prompt.requiresTextSelection === '1')) {
                        askUserToSelectText();

                        return;
                    }

                    let dom = tinymce.activeEditor.dom;
                    let $ = tinymce.dom.DomQuery;

                    const loadingSpinnerId = await addAutocompleteContainer(vixai.prompts[operation].generatedTextPlacement);

                    // remove selection
                    editor.selection.collapse();
                    let spinner = dom.select('#' + loadingSpinnerId);

                    let autocompletedText = '';
                    try {
                        autocompletedText = await doAutocompleteRequest(operation, selectedText, vixai.selectedLanguage);
                        autocompletedText = autocompletedText.replace(/\n/g, '<br/>');
                    } catch (error) {
                        // remove the spinner from the editor
                        $(spinner).remove();

                        alert('An API error occurred with the following response body: \n\n' + error.message);
                        return;
                    }

                    // remove the class and id from the spinner
                    $(spinner).removeAttr('class');
                    $(spinner).removeAttr('id');

                    // add the autocompleted text with animation slide down in css
                    $(spinner).html(autocompletedText);

                    // add undo level
                    editor.undoManager.add();
                }
            });
        }

        editor.addButton('vixai_classic_button_text', {
            type: 'menubutton',
            tooltip: 'تولید محتوا با هوش مصنوعی',
            image: vixai.pluginUrl + 'includes/icons/vixai.png',
            classes: 'vixai-classic-main-button',
            menu: menu
        });

        const addAutocompleteContainer = async function (placement) {
            let dom = tinymce.activeEditor.dom;
            let $ = tinymce.dom.DomQuery;
            const loadingSpinnerId = Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15);

            let selectionRange = editor.selection.getRng();
            if (placement === 'below') {
                let selectedNode = editor.selection.getEnd();

                let spinnerHtml = createLoadingSpinner(
                    selectedNode,
                    placement,
                    loadingSpinnerId,
                )
                let spinnerDom = $(spinnerHtml)[0];

                let parentNode = selectionRange.endContainer.parentNode;
                // if parent node is li then we need to create a new li
                if (parentNode.tagName.toLowerCase() === 'li') {
                    $(selectedNode).after(spinnerDom);
                } else if (selectedNode.textContent) {
                    selectionRange.collapse(false);
                    selectionRange.insertNode(spinnerDom);
                    editor.selection.collapse();
                } else {
                    $(selectedNode).after(spinnerDom);
                }

            } else { // above
                let selectedNode = editor.selection.getStart();
                let spinnerHtml = createLoadingSpinner(
                    selectedNode,
                    placement,
                    loadingSpinnerId,
                )
                let spinnerDom = $(spinnerHtml)[0];

                let parentNode = selectionRange.startContainer.parentNode;
                // if parent node is li then we need to create a new li
                if (parentNode.tagName.toLowerCase() === 'li') {
                    $(selectedNode).before(spinnerDom);
                } else if (selectedNode.textContent) {
                    selectionRange.collapse(true);
                    selectionRange.insertNode(spinnerDom);
                    editor.selection.collapse();
                } else {
                    $(selectedNode).before(spinnerDom);
                }
            }

            // add undo level
            editor.undoManager.add();

            return loadingSpinnerId;
        }

        const createLoadingSpinner = function (selectedNode, placement, loadingSpinnerId) {

            let spinnerHtml = '';
            if (['li'].includes(selectedNode.tagName.toLowerCase())) {
                spinnerHtml = '<' + selectedNode.tagName + ' id="' + loadingSpinnerId + '" class="vixai-mce-loading">&nbsp;</' + selectedNode.tagName + '>';
            } else {
                spinnerHtml = '<p id="' + loadingSpinnerId + '" class="vixai-mce-loading">&nbsp;</p>';
            }

            return spinnerHtml;
        }

        const doAutocompleteRequest = async function (requestType, text, selectedLanguage) {
            const siteUrl = vixai.siteUrl
            const nonce = vixai.nonce
            const response = await fetch(siteUrl + "/?rest_route=/vixai/openai/v1/autocomplete&type=" + requestType, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'X-WP-Nonce': nonce,
                },
                body: JSON.stringify({
                    text: text,
                    language: selectedLanguage,
                })
            }).catch(async error => {
                throw new Error(await error.text());
            })

            if (!response.ok) {
                throw new Error(await response.text());
            }

            const data = await response.json();

            return data.text
        }


        ///////////////////////////////////////////////////////
        // images
        ///////////////////////////////////////////////////////

        let imageGenerationOptions = vixai.imageGenerationOptions;
        let imageMenu = [];

        for (let size in imageGenerationOptions.sizes) {
            let resolution = imageGenerationOptions.sizes[size];
            for (let count of imageGenerationOptions.counts) {
                console.log(vixai.pluginUrl + 'includes/icons/image-' + size + '.svg')
                imageMenu.push({
                    text: count + ' x ' + tinymce.util.I18n.translate(size + ' تصویر') + ' (' + resolution + ')',
                    image: vixai.pluginUrl + 'includes/icons/image-' + size + '.svg',
                    classes: 'vixai-classic-button',
                    onclick: async function () {
                        if (isvixaiProperlyConfigured() === false) {
                            return;
                        }

                        let selectedText = editor.selection.getContent({format: 'text'});
                        if (selectedText === '') {
                            askUserToSelectText();

                            return;
                        }

                        let dom = tinymce.activeEditor.dom;
                        let $ = tinymce.dom.DomQuery;

                        const loadingSpinnerId = await addAutocompleteContainer('above');
                        let spinner = dom.select('#' + loadingSpinnerId);

                        try {
                            let result = await doImageGenerationRequest(count, size, selectedText, vixai.selectedLanguage);
                            let imagesHtml = '';
                            for (let image of result.images) {
                                let imageElementHtml = '<p><img data-mce-src="' + image.url + '" src="' + image.url + '" class="wp-image-' + image.id + '" width="300"  /></p>';
                                imagesHtml += imageElementHtml;
                            }

                            $(spinner).replaceWith(imagesHtml);

                        } catch (error) {
                            // remove the spinner from the editor
                            $(spinner).remove();

                            alert('An API error occurred with the following response body: \n\n' + error.message);
                        }
                    }
                });
            }
        }

        if (imageMenu.length > 0) {
            editor.addButton('vixai_classic_button_images', {
                type: 'menubutton',
                tooltip: 'ساخت تصویر با استفاده از هوش مصنوعی',
                image: vixai.pluginUrl + 'includes/icons/vixai-image.png',
                classes: 'vixai-classic-main-button',
                menu: imageMenu,
            });
        }

        const doImageGenerationRequest = async function (imageCount, imageSize, text, selectedLanguage) {
            const siteUrl = vixai.siteUrl
            const nonce = vixai.nonce
            const response = await fetch(siteUrl + "/?rest_route=/vixai/openai/v1/generate-images&count=" + imageCount + "&size=" + imageSize, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'X-WP-Nonce': nonce,
                },
                body: JSON.stringify({
                    text: text,
                    language: selectedLanguage,
                })
            }).catch(async error => {
                throw new Error(await error.text());
            })

            if (!response.ok) {
                throw new Error(await response.text());
            }

            const data = await response.json()

            return data
        }


        ///////////////////////////////////////////////////////
        // common
        ///////////////////////////////////////////////////////

        const askUserToSelectText = function (prompt) {
            tinymce.activeEditor.windowManager.alert("این گزینه به انتخاب متن نیاز دارد. لطفاً متنی را انتخاب کنید و دوباره امتحان کنید.");
        }

        const isvixaiProperlyConfigured = function () {
            if (vixai.isOpenAIKeyValid === false || vixai.isOpenAIKeyValid === 0 || vixai.isOpenAIKeyValid === '0' || vixai.isOpenAIKeyValid === null) {
                tinymce.activeEditor.windowManager.confirm("به نظر می رسد که vixai به درستی پیکربندی نشده است. آیا می خواهید آن را اکنون پیکربندی کنید؟", function(ok) {
                    if (ok) {
                        window.location.href = vixai.siteUrl + '/wp-admin/options-general.php?page=vixai';
                    }
                });

                return false;
            }

            return true;
        }
    });
});
