"use strict";

jQuery(function($) {

    $("#vixai-repurposer-form").submit(function (event) {
        event.preventDefault();

        submitForm($("#vixai-repurposer-generate"));
    });

    // on defocus of vixai-repurposer-url, check if it is a valid URL
    $("#vixai-repurposer-url").blur(function () {
        let url = $(this).val();
        if (url === '') {
            return;
        }

        if (!isValidUrl(url)) {
            alert('Please enter a valid URL. (including http:// or https://)');
            return;
        }

        // of the url equals the data-url attribute, then we have already extracted the text
        if (url === $(this).data('url')) {
            return;
        }

        //store the url as a data attribute
        $(this).data('url', url);

        // append a spinner to the text box
        let spinner = $('<span class="spinner-border spinner-border-sm float-end m-2" role="status" aria-hidden="true"></span>');
        $(this).after(spinner);

        // extract text using ajax request
        $.ajax({
            type: "POST",
            url: vixai.siteUrl + '/?rest_route=/vixai/repurposer/v1/extract-text',
            data: JSON.stringify({url: url}),
            dataType: "json",
            encode: true,
            headers: {
                'Content-Type': 'application/json',
                'X-WP-Nonce': vixai.nonce,
            },
        }).success(function (response) {
            spinner.remove();
            $("#vixai-repurposer-extracted-text").val(response.content);
            $(".vixai-repurposer-extracted-text-container").show(100);
        }).fail(function (response) {
            spinner.remove();
            alert('Error: ' + response.responseText);
        });
    });

    $(".vixai-repurposer-jobs-delete").on('click', function (event) {
        event.preventDefault();
        if (confirm($(this).data('confirm-message'))) {
            let deleteUrl = $(this).attr('href');
            $.ajax({
                type: "POST",
                url: deleteUrl,
                data: JSON.stringify({
                    id: $(this).data('id'),
                }),
                dataType: "json",
                encode: true,
                headers: {
                    'Content-Type': 'application/json',
                    'X-WP-Nonce': vixai.nonce,
                },
            }).success(function (response) {
                // refresh the page
                location.reload();
            }).fail(function (response) {
                alert('Error: ' + response.responseText);
            });
        }
    });

    const submitForm = function (button) {
        if (!isProperlyConfigured()) {
            alert('Please enter a valid OpenAI API key in the settings page.');
            return;
        }

        if ($("#vixai-repurposer-url").val() === '') {
            alert($('#vixai-repurposer-url').data('validation-message'));
            return;
        }

        button.prepend('<span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>');
        // disable the button
        button.prop('disabled', true);

        let prompts = {};
        $(".vixai-repurposer-prompt").each(function () {
            prompts[$(this).data('prompt-id')] = $(this).val();
        });

        let formData = {
            url: $("#vixai-repurposer-url").val(),
            keywords: $("#vixai-repurposer-seo-keywords").val(),
            include_featured_image: $("#vixai-repurposer-include-featured-image").is(':checked'),
            number_of_articles: $("#vixai-repurposer-articles").val(),
            post_type: $("#vixai-repurposer-post-type").val(),
            post_status: $("#vixai-repurposer-post-status").val(),
            post_category: $("#vixai-repurposer-post-category").val(),
            prompts: prompts,
        };

        // if vixai-repurposer-extracted-text has text, then send it as well
        if ($("#vixai-repurposer-extracted-text").val() !== '') {
            formData.extracted_text = $("#vixai-repurposer-extracted-text").val();
        }

        let url = $('#vixai-repurposer-form').attr('action');

        $.ajax({
            type: "POST",
            url: url,
            data: JSON.stringify(formData),
            dataType: "json",
            encode: true,
            headers: {
                'Content-Type': 'application/json',
                'X-WP-Nonce': vixai.nonce,
            },
        }).success(function (response) {
            button.find('.spinner-border').remove();
            button.prop('disabled', false);

            showToast(
                translate('Created'),
                translate('Repurpose job') +
                ' <a href="' + response.url + '">' +
                translate('created Successfully.') +
                '</a>');

        }).fail(function (response) {
            alert('Error: ' + response.responseText);
            button.find('.spinner-border').remove();
            button.prop('disabled', false);
        });
    }

    const translate = function (text) {
        let translations = JSON.parse($('#vixai-repurposer-translations').val());
        if (translations[text]) {
            return translations[text];
        }

        return text;
    }

    const showToast = function (title, message) {

        let container = $('<div class="toast-container position-fixed bottom-0 end-0 p-3">');
        let toast = $('<div class="toast" role="alert" aria-live="assertive" aria-atomic="true">');
        let toastHeader = $('<div class="toast-header">');
        toastHeader.append('<strong class="me-auto">' + title + '</strong>');
        toastHeader.append('<button type="button" class="btn-close" data-bs-dismiss="toast" aria-label="Close"></button>');
        let toastBody = $('<div class="toast-body">');

        toastBody.append(message);
        toast.append(toastHeader);
        toast.append(toastBody);
        container.append(toast);
        $('body').append(container);

        // show the toast
        let toastEl = new bootstrap.Toast(toast);
        toastEl.show();
    }

    const isProperlyConfigured = function () {
        if (vixai.isOpenAIKeyValid === undefined || vixai.isOpenAIKeyValid === "0" || vixai.isOpenAIKeyValid === "" || vixai.isOpenAIKeyValid === false) {
            return false;
        }

        return true;
    }

    const isValidUrl = function (url) {
        try {
            new URL(url);
        } catch (_) {
            return false;
        }

        return true;
    }
});
